"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const error_1 = require("graphql/error");
const language_1 = require("graphql/language");
// we will store the string as `XXX 1.234` with digits up to the digits limit.
const moneyRe = /^([A-Z]{3}) [0-9]+(|.[0-9]+)$/;
function parseMoneyString(input) {
    const match = input.match(moneyRe);
    if (!match) {
        throw new TypeError("Invalid Money Format");
    }
    const [, code, majorS, minorS] = match;
    const major = safeInt(parseInt(majorS, 10));
    const minor = minorS.length > 1 ? safeInt(parseInt(minorS.slice(1), 10)) : 0;
    return [code, major, minor];
}
function formatMoneyString(curr, minor) {
    if (curr.digits > 0) {
        // precision here is as much as we get with 6dp.
        return `${curr.code} ${minor.toFixed(6).replace(/(\.[0-9]+)0+$/, "$1")}`;
    }
    const n = minor / 10 ** curr.digits;
    return `${curr.code} ${n.toFixed(curr.digits)}`;
}
const safeInt = (value) => {
    if (!Number.isFinite(value)) {
        throw new TypeError("value is not finite");
    }
    if (Number.isNaN(value)) {
        throw new TypeError("value is not a number");
    }
    if (!Number.isSafeInteger(value)) {
        throw new TypeError("value is not a (safe) integer");
    }
    return value;
};
exports.customMoneyScalar = (currencyList, defaultCurrency = currencyList[0]) => {
    const currencyMap = new Map(currencyList.map(c => [c.code, c]));
    const serialize = (value) => {
        switch (typeof value) {
            case "string":
                // must be a valid string!
                const [code, major, minor] = parseMoneyString(value);
                const currency = currencyMap.get(code);
                if (!currency) {
                    throw new TypeError("Unknown Currency: " + code);
                }
                if (currency.digits < 0) {
                    // less than zero can have arbitrary precision
                    // but we must reconstruct the actual number for formatting.
                    return formatMoneyString(currency, parseFloat(`${major}.${minor}`));
                }
                else {
                    if (minor >= 10 ** currency.digits) {
                        throw new TypeError("Sub-Minor Currency denominations are not allowed");
                    }
                }
                // OK. return canonical value.
                return formatMoneyString(currency, 10 ** currency.digits * major + minor);
            case "number":
                // default currency, in minor units.
                return formatMoneyString(defaultCurrency, value);
            default:
                throw new TypeError("Invalid value given for 'Money' value");
        }
    };
    const deserialize = (v) => {
        const [code, major, minor] = parseMoneyString(v);
        const currency = currencyMap.get(code);
        if (!currency) {
            throw new TypeError("Unknown Currency: " + code);
        }
        if (currency.digits < 0) {
            // less than zero can have arbitrary precision
            // but we must reconstruct the actual number for formatting.
            return {
                currency,
                value: parseFloat(`${major}.${minor}`)
            };
        }
        else {
            if (minor >= 10 ** currency.digits) {
                throw new TypeError("Sub-Minor Currency denominations are not allowed");
            }
        }
        return {
            currency,
            value: safeInt(major * 10 ** currency.digits + minor)
        };
    };
    return new graphql_1.GraphQLScalarType({
        name: "Money",
        description: "Currency field, supports: " + Array.from(currencyMap.keys()).join(", "),
        serialize,
        parseValue: serialize,
        parseLiteral: ast => {
            if (ast.kind !== language_1.Kind.STRING) {
                throw new error_1.GraphQLError(`Can only parse strings to money but got a: ${ast.kind}`);
            }
            return deserialize(ast.value);
        }
    });
};
exports.createCurrency = (code, digits = 2) => {
    const def = (v) => {
        return { currency: def, value: v };
    };
    def.code = code;
    def.digits = digits;
    def.format = (v) => formatMoneyString(def, v);
    return def;
};
// just a few currencies to start with
exports.Currencies = {
    GBP: exports.createCurrency("GBP"),
    EUR: exports.createCurrency("EUR"),
    USD: exports.createCurrency("USD"),
    CAD: exports.createCurrency("CAD"),
    CHF: exports.createCurrency("CHF"),
    JPY: exports.createCurrency("JPY", 0),
    IQD: exports.createCurrency("IQD", 3),
    XAU: exports.createCurrency("XAU", -1)
};
// needs some tests!
exports.MoneyScalar = exports.customMoneyScalar([exports.Currencies.GBP]);
//# sourceMappingURL=money.js.map